#! /usr/local/perl
#
#   Compare two AVR ROM files
#
sub usage {
    print STDERR "usage: $0 [-d] current.ROM new.ROM\n";
    exit(1);
}

$debug = 0;

if ($ARGV[0] eq '-d') {
	$debug = 1;
	shift;
}

&usage if ($ARGV[0] eq '');
&usage if ($ARGV[1] eq '');
&usage if ($ARGV[2] ne '');
$cur = $ARGV[0];
$new = $ARGV[1];
if (!open(CUR,"<$cur")) {
    print STDERR  "$0: can't open current ROM file $cur\n";
    exit(1);
}
if (!open(NEW,"<$new")) {
    print STDERR "$0: can't open new ROM file $new\n";
    exit(1);
}

# read current ROM status
#
#   000000:c88f^M
#   000001:c88d^M
#   000002:c88c^M
$linenum = 0;
while (<CUR>) {
	++$linenum;
	$line = $_;
	$line =~ s/[\r\n\s]*$//;
	if ($line =~ /^([0-9a-f]+)[:]([0-9a-f]+)$/i) {
		$ix = hex($1);
		$data[$ix] = hex($2);
	} else {
		print STDERR "bad ROM file input $cur($linenum): $line\n";
		exit(1);
	}
}

# read/check new ROM file
#
#   000000:c88f^M
#   000001:c88d^M
#   000002:c88c^M
$linenum = 0;
$num_mismatch = 0;
$num_checked = 0;
$text_mismatch = '';
while (<NEW>) {
	++$linenum;
	$line = $_;
	$line =~ s/[\r\n\s]*$//;
	if ($line =~ /^([0-9a-f]+)[:]([0-9a-f]+)$/i) {
		$ix = hex($1);
		$val = hex($2);
                ++$num_checked;
		if ($data[$ix] != $val) {
			# trailing 0xffff's are trimed from "old" ROM file
			if (!($data[$ix] eq '' && $val == 0xffff)) {
				print "ROM files $cur and $new mismatch at word $ix ($line, $data[$ix]!=$val)\n" if ($debug);
				++$num_mismatch;
			}
                        if ($text_mismatch eq '') {
				$text_mismatch = sprintf("0x%04x,0x%04x->0x%04x", $ix, $data[$ix], $val);
			}
		}
	} else {
		print STDERR "bad ROM file input $new($linenum): $line\n";
		exit(1);
	}
}
if ($num_mismatch > 0) {
	print "$num_mismatch changed locations in file $new (@$text_mismatch)\n";
        exit(0);
} else {
	print "ROM files $cur and $new match\n" if ($debug);
	print "No load required for file $new ($num_checked locations checked)\n";
	exit(10);
}
