#! /usr/local/perl
#
#  Convert Ethernet (MAC) address (dot notation) to "ether.inc"
#

sub usage {
	print STDERR "usage: $ego [-v] a.b.c.d.e.f\n";
	print STDERR "       $ego [-v] - < file\n";
	exit 1;
}

$ego = $0;
$ego =~ s/.*[\/\\]//;

$verbose = 0;
while ($ARGV[0] =~ /^-/ && $ARGV[0] ne '-') {
	$opt = shift;
	if ($opt eq '-v') {
		$verbose = 1;
	} else {
		&usage;
	}
}

$mac = shift;
&usage if ($mac eq '');

exit(1) if (($mac = &getmac($mac)) eq '');

(@a) = split(/\./,$mac) ;
$mac = sprintf("%d.%d.%d.%d.%d.%d", $a[0], $a[1], $a[2], $a[3], $a[4], $a[5]);

printf("\t.dw\t0x%02x%02x\t; Ethernet address $mac\n", $a[1], $a[0]);
printf("\t.dw\t0x%02x%02x\n", $a[3], $a[2]);
printf("\t.dw\t0x%02x%02x\n", $a[5], $a[4]);

print STDERR "$mac\n" if ($verbose);

#
#  Resolve MAC address
#  usage: exit(1) if (($mac = &getmac($mac)) eq '');
#         where $s is MAC address, '-', or @file
#
sub getmac {
    local($f) = @_;
    local($s);

    $mac = '';
    if ($f =~ /^[@]/ || $f eq '-') {
        if ($f ne '-') {
            $f =~ s/^[@]//;
            if (!open(MAC, "<$f")) {
                print STDERR "Can't open MAC address file \"$f\"!\n";
                return '';
            }
        }
        while(1) {
            if ($f ne '-') {
                last if (!($s = <MAC>));
            } else {
                last if (!($s = <>));
            }
            ##print STDERR "$s";
            $s =~ s/[\s\r\n]+$//;
            $s =~ s/\s*[#].*$//;
            next if ($s eq '');
            if ($s =~ /^\s*(\d+[.]\d+[.]\d+[.]\d+)[.\r\n\s\d]*$/) {
                $mac = $s;
            }
        }
    } else {
        $mac = $f;
    }
    ##print STDERR "MAC/NM=$mac\n";
    if ($mac =~ /^\d+[.]\d+[.]\d+[.]\d+[.]\d+[.]\d+$/) {
        return $mac;
    } else {
	print STDERR "Ethernet address ($mac) not of the form a.b.c.d.e.f!\n";
        return ('', '');
    }
}
