;--------------------------------------------------------------------------
;+
; **-pprintswi-output signed 16-bit integer in decimal
;
; pprintswi value
;
; inputs:
;   value = 16-bit word to convert
;
; outputs:
;   result printed (with pputc), right-justified in 6-character field
;
; working SRAM:
;   15 bytes (save/restores on stack)
;
;--------------------------------------------------------------------------
#define T1 (BP+0)
#define T2 (T1+2)
#define T3 (T2+2)
#define TEMPS_END (T3+5)
#define N_TEMPS (TEMPS_END-T1)

#define P1    TEMPS_END
#define P2    P1+2
#define P3    P2+2

#define DIG_BUF T3      /* uses 5 bytes for ASCII digit storage */
#define DIG_SGN T2
#define DIG_CNT T1
#define DIG_CH  DIG_CNT
#define DIG_PTR P3
#define DIG_LST DEC_REM
#define DEC_REM P2        /* must be adjacent in RAM (high) */
#define DEC_VAL P1        /* must be adjacent in RAM (low) */

.text
.global pprintswi
pprintswi: pcode_routine  1
    rcall pip_setup
    pjump printint

.global pprintuwi
pprintuwi: pcode_routine  1
    rcall pip_setup
    pjump printint_unsigned

#if 0
.section seeprom
#else
.text
#endif

printint_unsigned:
    penter N_TEMPS
    pmovwi DIG_SGN,' '                  ; set default sign
    pjump printint0

printint:
    penter N_TEMPS
    pmovwi DIG_SGN,' '                  ; set default sign
    pbitwi DEC_VAL,0x8000               ; check the sign
    pjumpeq printint0                   ; positive - start conversion
    pnegw DEC_VAL                       ; negate it
    pmovwi DIG_SGN,'-'                  ; change sign to negative

printint0:
    pmovwi DIG_PTR,DIG_BUF              ; setup output buffer pointer
    pmovwi DIG_CNT,5                    ; we will do 5 bytes
printint1:
    pdiv DEC_VAL,[DEC_VAL],10           ; do next digit
    paddwi DEC_REM,'0'                  ; convert digit for display
    pmovbi [DIG_PTR],[byte DEC_REM]     ; save digit
    pincw DIG_PTR                       ; bump past digit just stored
    pdecw DIG_CNT                       ; decrement count
    pjumpne printint1                   ; more to do if non-zero

    pclrw DIG_CH                        ; setup to output digits
    pmovwi DIG_LST,' '                  ; make "last character" a space
printint2:
    pdecw DIG_PTR                       ; point to next digit
    pmovb DIG_CH,[DIG_PTR]              ; get next digit
    pcmpwi DIG_CH,'0'                   ; is it '0'?
    pjumpne printint4                       ; no - may need sign char
    pcmpwi DIG_LST,' '                  ; last char ' '?
    pjumpne printint3                       ; no - continue as normal
    pcmpwi DIG_PTR,DIG_BUF              ; this is the very last digit?
    pjumpeq printint4                       ; yes - check on sign
    pmovwi DIG_CH,' '                   ; output a ' ' instead

printint3:
    pputcb DIG_CH                       ; output the current digit
    pmovb DIG_LST,DIG_CH                ; save last digit output
    pcmpwi DIG_PTR,DIG_BUF              ; check if just did last one
    pjumpne printint2                      ; no - do the next digit

    pretn N_TEMPS                       ; fix stack and return to caller

printint4:
    pcmpwi DIG_SGN,0                    ; have we output sign char?
    pjumpeq printint3                       ; no - continue as normal
    pputcb DIG_SGN                      ; output sign
    pclrw DIG_SGN                       ; zap sign character
    pjump printint3                     ; continue as normal

