;--------------------------------------------------------------------------
;+
; **-purl2scmp-string compare with receive packet buffer.
;
; purl2scmp poffset,string
;
; inputs:
;   poffset = sram address with offset in receive packet data area
;             or address in seeprom (if bit 15 set)
;   string = address of string (null terminated) in flash or seeprom
;
; outputs:
;   pcode flags Z=1 : entire string matches input buffer
;               Z=0 : strings mismatch
;   [poffset] = if Z=0, original value
;               if Z=1, incremented by strlen(string)
;-
;--------------------------------------------------------------------------

.text

#define T1 (BP+0)
#define T2 (T1+2)
#define T3 (T2+2)
#define T4 (T3+2)
#define TEMPS_END (T4+2)
#define N_TEMPS (TEMPS_END-T1)

#define P1    TEMPS_END
#define P2    P1+2
#define P3    P2+2

#define poffset    P1          /* must be adjacent in RAM! */
#define SCMP_S     P2
#define SCMP_OFF   P3          /* scratch */
#define SCMP_CNT   T2
#define SCMP_R2S   T3     /* purl2scmp character here */
#define SCMP_STRN  T4     /* string character here */

#ifdef DEBUG
#define D
#else
#define D ;
#endif

    .global purl2scmp
purl2scmp: pcode_routine 2

    rcall pip_setup_r10r12reloc
    penter N_TEMPS

    pmovw SCMP_OFF,[poffset]

    D pserial_debug
    D pprintv "purl2scmp (poffset=",[poffset]
    D pprintv ",SCMP_OFF=",[SCMP_OFF]
    D pprintv ",SCMP_S=",[SCMP_S]
    D pprint ")\r\n"

    pmovwi SCMP_CNT,0
purl2scmp_loop:
    D pprint "[";
    purl2s SCMP_R2S,[SCMP_OFF],1     ; next character from packet buffer
    pjumpne purl2scmp_nomatch          ; exit with these flags if none
    D pputcb SCMP_R2S
    D pprint "|";
    pgetsb SCMP_STRN,[SCMP_S],[SCMP_CNT] ; and next from string
    D pputcb SCMP_STRN
    D pprint "]";
    pcmpbi SCMP_R2S,[SCMP_STRN]      ; compare characters
    pjumpne purl2scmp_nomatch          ; exit with these flags if not zero
    D pprint "=";
    pcmpbi SCMP_STRN,0               ; check for NULL char from string?
    pjumpeq purl2scmp_match            ; we hit a NULL in string (match)
    pincw SCMP_OFF                   ; bump buffer address
    pincw SCMP_CNT                   ; bump string offset
    pjump purl2scmp_loop               ; not NULL...check next character

purl2scmp_nomatch:
    D pprint "?";
    pcmpbi SCMP_STRN,0               ; check for NULL char in string?
    pjumpne purl2scmp_nomatch2         ; not NULL...return mismatch
purl2scmp_match:
    D pprint "*";
    pcmpwi SCMP_OFF,[SCMP_OFF]       ; NULL...show match
    pmovwi [poffset],[SCMP_OFF]        ; ...and update offset
    pjump purl2scmp_exit

purl2scmp_nomatch2:
    D pprint "X\r\n";
    pcmpwi SCMP_S,0                  ; set flags (always will mismatch)

purl2scmp_exit:

    D pprintv "&poffset=",poffset
    D pprintv " poffset=",[poffset]
    D pprintv " *poffset=>",[SCMP_OFF]
    D pcrlf

    pretn N_TEMPS

